//===========================================================================
// PROJECT: StatusIcon
// DESC   : Show a icon on the taskbar's tray.
// VERSION : 1.02
//  Copyright (c) 1997,1998 by Eddie Carpenter (ecarpenter@itex.co.za)
//
//   You are hereby granted a royalty-free unlimited distribution
//   license to compile the components in this source file into
//   your applications.
//
//   This source file may be freely distributed through online networks
//   so long as no modifications are made to this file and no fee is
//   charged to obtain this file, other than normal online connection
//   charges.
//
//   These components may NOT be distributed as source code or
//   compiled DCU on diskette, CDRom, or as part of a
//   product without prior written consent of the author.
//
//   All rights not explicitly granted here are reserved by the author.
//
//   Change History
//
//   Version 1.02
//      Fixed bug where the szTip property of the NOTIFYICONDATA structure is
//      only 64 characters and I tried to move more than 64 characters that to
//      the field.
//      Thank you Gatan Delaloye for reporting this bug.
//
//   Version 1.01
//      Change Hint Property to update the icon information when changed.
//
//   Version 1.00
//      Initial Release
//===========================================================================

#include <vcl\vcl.h>
#pragma hdrstop
#include <dsgnintf.hpp>
#include <memory>
#include <imglist.hpp>
#include "stdreg.hpp"
#include "StatusIcon.h"
#pragma resource "*.dfm"
//-------------------------------------------------------[ ValidCtrCheck ]---
static inline TStatusIcon *ValidCtrCheck()
{
  return new TStatusIcon(NULL);
}
//---------------------------------------------------------------------------
namespace Statusicon
{
        void __fastcall Register()
        {
                 TComponentClass classes[1] = {__classid(TStatusIcon)};
                 RegisterComponents("Samples", classes, 0);
                 RegisterPropertyEditor(__typeinfo(TAboutProperty),
                         __classid(TStatusIcon),
                         "ABOUT",
                         __classid(TAboutProperty));

        }
}


//------------------------------------------------------------[ TAboutDlg ]---
__fastcall TAboutDlg::TAboutDlg(TComponent* Owner)
   : TForm(Owner)
{
}

//-----------------------------------------------------------[ OKBtnClick ]---
void __fastcall TAboutDlg::OKBtnClick(TObject *Sender)
{
  Close();
}//OKBtnClick

//-------------------------------------------------------[ TAboutProperty ]---
__fastcall TAboutProperty::TAboutProperty(void)
   : TClassProperty()
{
} //TAboutProperty

//------------------------------------------------------[ ~TAboutProperty ]---
__fastcall TAboutProperty::~TAboutProperty(void)
{
}//~TAboutProperty

//----------------------------------------------------------------[ Edit ]---
void __fastcall TAboutProperty::Edit()
{
  TAboutDlg *Form;
  Form = new TAboutDlg(Application);
  Form->ShowModal();
  delete Form;
}//Edit

//-------------------------------------------------------[ GetAttributes ]---
TPropertyAttributes __fastcall TAboutProperty::GetAttributes(void)
{
  return (TPropertyAttributes() << paDialog << paReadOnly);
}//GetAttributes

//------------------------------------------------------------[ GetValue ]---
AnsiString __fastcall TAboutProperty::GetValue()
{
  return "(About info)";
}//GetValue

//---------------------------------------------------------[ TStatusIcon ]---
__fastcall TStatusIcon::TStatusIcon(TComponent* Owner)
        : TComponent(Owner)
{
  WNDCLASS Temp;
  bool Registered;
  WNDCLASS WinClass;

  WinClass.style = CS_DBLCLKS;
  WinClass.lpfnWndProc = DefWindowProc;
  WinClass.cbClsExtra = 0;
  WinClass.cbWndExtra = 0;
  WinClass.hInstance = HInstance;
  WinClass.hIcon = 0;
  WinClass.hCursor = 0;
  WinClass.hbrBackground =  0;
  WinClass.lpszMenuName = NULL;
  WinClass.lpszClassName = "TStatusIcon";

  Registered = GetClassInfo(HInstance, WinClass.lpszClassName, &Temp);
  if (!Registered || Temp.lpfnWndProc != DefWindowProc)
  {
    if (Registered)
      UnregisterClass(WinClass.lpszClassName, HInstance);
    RegisterClass(&WinClass);
  }//if

  FHandle = CreateWindowEx(0,WinClass.lpszClassName, "",
     0, 0, 0, 0,0, 0, 0, HInstance, NULL);

  SetWindowLong(FHandle, GWL_WNDPROC, (long)MakeObjectInstance(WndProc));

  FIcon = new TIcon();
  DoubleClick = false;
}//TStatusIcon

//--------------------------------------------------------[ ~TStatusIcon ]---
__fastcall TStatusIcon::~TStatusIcon(void)
{
  WNDPROC WP;

  FEnabled = false;
  UpdateIcon();
  delete FIcon;

  WP = (WNDPROC )GetWindowLong(FHandle, GWL_WNDPROC);
  DestroyWindow(FHandle);
  FreeObjectInstance(WP);
}//TStatusIcon

//------------------------------------------------------------[ SetIcons ]---
void __fastcall TStatusIcon::SetIcons(TImageList *Icons)
{
  FIcons = Icons;
  UpdateIcon();
}//SetIcons

//--------------------------------------------------------[ SetIconIndex ]---
void __fastcall TStatusIcon::SetIconIndex(int Index)
{
  if (Index > FIcons->Count)
    Index = FIcons->Count;

  FIconIndex = Index;
  FIcons->GetIcon(FIconIndex, FIcon);
  UpdateIcon();
}//SetIconIndex

//--------------------------------------------------------[ SetShowHint ]---
void __fastcall TStatusIcon::SetShowHint(bool Value)
{
  FShowHint = Value;
  UpdateIcon();
}//SetShowHint

//---------------------------------------------------------[ SetEnabled ]---
void __fastcall TStatusIcon::SetEnabled(bool Value)
{
  FEnabled = Value;
  UpdateIcon();
}//SetEnabled

//--------------------------------------------------------[ SetPopupMenu ]---
void __fastcall TStatusIcon::SetPopupMenu(TPopupMenu *Value)
{
  FPopupMenu = Value;
  if (Value != NULL)
    Value->FreeNotification(this);
}//SetPopupMenu


//--------------------------------------------------------[ SetHideAppl ]---
void __fastcall TStatusIcon::SetHideAppl(bool Value)
{
  FHideAppl = Value;

  if ( ComponentState.Contains(csDesigning) )
    return;
      
  if (FHideAppl)
    ShowWindowAsync(Application->Handle,SW_HIDE);
  else
    ShowWindowAsync(Application->Handle,SW_SHOW);
}//SetHideAppl

//----------------------------------------------------------[ SetHint ]---
void __fastcall TStatusIcon::SetHint(AnsiString Value)
{
  FHint = Value;
  UpdateIcon();
}//SetHint

//------------------------------------------------------------[ WndProc ]---
void __fastcall TStatusIcon::WndProc(Messages::TMessage &Message)
{
  TMessage Msg;
  TPoint   MousePos;

  if (Message.Msg == WM_TRAYICON && Message.WParam == 1)
  {
    GetCursorPos(&MousePos);
    Msg.Msg = Message.LParam;
    Msg.LParamLo = short(MousePos.x);
    Msg.LParamHi = short(MousePos.y);
    Msg.WParam = 0;

    Dispatch(&Msg);
    Message.Result = Msg.Result;
  }//if
  else
    Message.Result = DefWindowProc(FHandle, Message.Msg, Message.WParam, Message.LParam);
}//WndProc

//-----------------------------------------------------[ WMLButtonDblClk ]---
void __fastcall TStatusIcon::WMLButtonDblClk(TWMLButtonDblClk &Message)
{
  TShiftState Shift;

  DblClick();
  Shift << ssDouble;
  MouseDown(Message, mbLeft, Shift);
}//WMLButtonDblClk

//--------------------------------------------------------[ WMLButtonDown ]---
void __fastcall TStatusIcon::WMLButtonDown(TWMLButtonDown &Message)
{
  TShiftState Shift;
  MouseDown(Message, mbLeft, Shift);
}//WMLButtonDown

//--------------------------------------------------------[ WMRButtonDown ]---
void __fastcall TStatusIcon::WMRButtonDown(TWMRButtonDown &Message)
{
  TShiftState Shift;
  MouseDown(Message, mbRight, Shift);
}//WMRButtonDown

//--------------------------------------------------------[ WMMButtonDown ]---
void __fastcall TStatusIcon::WMMButtonDown(TWMMButtonDown &Message)
{
  TShiftState Shift;
  MouseDown(Message, mbMiddle, Shift);
}//WMMButtonDown

//----------------------------------------------------------[ WMMouseMove ]---
void __fastcall TStatusIcon::WMMouseMove(TWMMouseMove &Message)
{
  TShiftState Shift;
  MouseMove(Shift, Message.XPos, Message.YPos);
}//WMMouseMove

//----------------------------------------------------------[ WMLButtonUp ]---
void __fastcall TStatusIcon::WMLButtonUp(TWMLButtonUp &Message)
{
  Click();
  MouseUp(Message, mbLeft);
}//WMLButtonUp

//----------------------------------------------------------[ WMRButtonUp ]---
void __fastcall TStatusIcon::WMRButtonUp(TWMRButtonUp &Message)
{
  MouseUp(Message, mbRight);
  if (PopupMenu != NULL&& PopupMenu->AutoPopup)
  {
    SetForegroundWindow (Application->Handle);
    Application->ProcessMessages();
    PopupMenu->PopupComponent = this;
    PopupMenu->Popup(Message.XPos,Message.YPos);
  }//if
}//WMRButtonUp

//----------------------------------------------------------[ WMMButtonUp ]---
void __fastcall TStatusIcon::WMMButtonUp(TWMMButtonUp &Message)
{
  MouseUp(Message, mbMiddle);
}//WMMButtonUp

//---------------------------------------------------------------[ Click ]---
void TStatusIcon::Click()
{
  if (!DoubleClick  && FOnClick)
    FOnClick(this);
  DoubleClick = false;
}//Click

//------------------------------------------------------------[ DblClick ]---
void TStatusIcon::DblClick()
{
  DoubleClick = true;
  if (FOnDblClick)
    FOnDblClick(this);
}//DblClick

//------------------------------------------------------------[ MouseMove ]---
void TStatusIcon::MouseMove(TShiftState Shift,int  X, int Y)
{
  if (FOnMouseMove)
     FOnMouseMove(this, Shift, X, Y);
}//MouseMove

//-------------------------------------------------------------[ MouseUp ]---
void TStatusIcon::MouseUp(TWMMouse Message,TMouseButton  Button)
{
  TShiftState Shift;

  if (FOnMouseUp)
    FOnMouseUp(this, Button, Shift, Message.XPos, Message.YPos);
}//MouseUp

//----------------------------------------------------------[ MouseDown ]---
void TStatusIcon::MouseDown(TWMMouse Message, TMouseButton Button, TShiftState Shift)
{
  if (FOnMouseDown)
      FOnMouseDown(this, Button, Shift, Message.XPos, Message.YPos);
}//MouseDown

//----------------------------------------------------------[ UpdateIcon ]---
void TStatusIcon::UpdateIcon()
{
  if ( ComponentState.Contains(csDesigning) )
    return;

  if (FIcons != NULL)
    FIcons->GetIcon(FIconIndex, FIcon);

  if (FEnabled == false)
  {
    if (nid.hWnd != 0)
      Shell_NotifyIcon(NIM_DELETE, &nid);
    nid.hWnd = 0;
  }//if
  else
  {
    nid.cbSize = sizeof(NOTIFYICONDATA);
    nid.uID = 1;
    nid.uFlags = NIF_MESSAGE + NIF_ICON + NIF_TIP;

    if (FIcon != NULL)
      nid.hIcon  = FIcon->Handle;

    if (FShowHint)
      strncpy(nid.szTip, Hint.c_str(), 64);

    nid.uCallbackMessage = WM_TRAYICON;
    if (nid.hWnd == Handle)
        Shell_NotifyIcon(NIM_MODIFY, &nid);
    else
    {
      nid.hWnd = Handle;
      Shell_NotifyIcon(NIM_ADD, &nid);
    }//else
  }//else
}//UpdateIcon
//---------------------------------------------------------------------------
//
//                               T H E   E N D
//
//---------------------------------------------------------------------------

